# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_common.ipynb.

# %% auto 0
__all__ = ['get_image_grid', 'show_grid', 'normalize_axes', 'image_grid', 'image_size', 'image_sizes_hist', 'one_batch_with_idxs',
           'show_tfms']

# %% ../nbs/00_common.ipynb 3
#TODO: optimize imports
from fastai.vision.all import *
from mpl_toolkits.axes_grid1 import ImageGrid

# %% ../nbs/00_common.ipynb 12
@delegates(ImageGrid)
def get_image_grid(
    nitems=None,
    ncols=3,
    nrows=None,
    pad=0.0,
    figsize=None,
    **kwargs
):
    "Creates axes optimized for displaying images."
    nrows = nrows or int(np.ceil(nitems / ncols))
    figsize = figsize or (14, 4 * nrows)

    fig = plt.figure(figsize=figsize)
    grid = ImageGrid(fig, 111, (nrows, ncols), axes_pad=pad, **kwargs)
    
    return fig, grid

# %% ../nbs/00_common.ipynb 13
def show_grid(grid, shows, show=True):
    for ax, show in zip(grid, shows): show(ctx=ax)
    if show: plt.show()

# %% ../nbs/00_common.ipynb 14
def normalize_axes(grid, xmax, ymax):
    "Expand all axes to have the same dimensions."
    for ax in grid:
        ax.set_xlim(right=xmax)
        ax.set_ylim(bottom=ymax)

# %% ../nbs/00_common.ipynb 15
@delegates(get_image_grid, but=['nitems'])
def image_grid(shows, show=True, xmax=None, ymax=None, **kwargs):
    "Quickly plot a grid of images."
    fig, grid = get_image_grid(nitems=len(shows), **kwargs)
    show_grid(grid, shows, show=show)
    normalize_axes(grid, xmax, ymax)
    return fig, grid

# %% ../nbs/00_common.ipynb 17
def image_size(image_file):
    with Image.open(str(image_file)) as image:
        return image.size

# %% ../nbs/00_common.ipynb 18
def image_sizes_hist(
    image_files # Sequence of image filepaths.
):
    "Plot a histogram of widths and heights."
    sizes = parallel(image_size, image_files, progress=progress_bar)
    widths, heights = zip(*sizes)
    plt.hist(widths, label='width')
    plt.hist(heights, label='heights')
    plt.legend()

# %% ../nbs/00_common.ipynb 21
def one_batch_with_idxs(
    dl, # A `DataLoader` instance. Commonly `dls.train` or `dls.valid`
    unique_idx=None, # If specified, show only images with index `unique_idx`.
):
    "Show original and transformed version of images."
    old_get_idxs = dl.get_idxs

    idxs = dl.get_idxs() if unique_idx is None else [unique_idx] * len(dl.items)
    dl.get_idxs = lambda: idxs
    x, y = dl.one_batch()

    dl.get_idxs = old_get_idxs
    
    return (x, y), idxs

# %% ../nbs/00_common.ipynb 22
@delegates(image_grid)
def show_tfms(
    dl, # A `DataLoader` instance. Commonly `dls.train` or `dls.valid`
    unique_idx=None, # If specified, show only images with index `unique_idx`.
    max_n=5, # Maximum number of samples.
    pad=0.2,
    ncols=2,
    **kwargs,
):
    "Show original and transformed version of images."
    (xs, ys), idxs = one_batch_with_idxs(dl, unique_idx=unique_idx)
    
    shows = []
    xmax, ymax = 0, 0 # Has to be passed to show grid so images are not cut when displayed
    for i, x, idx in zip(range(max_n), xs, idxs):
        orig = dl.dataset[idx]
        image = orig[0]
        xmax = max(xmax, x.shape[2], image.shape[1])
        ymax = max(ymax, x.shape[1], image.shape[0])
        
        def _s(ctx, orig=orig, idx=idx): return dl.dataset.show(orig, ctx=show_title(idx, ctx=ctx))
        shows.append(_s)

    fig, grid = get_image_grid(nrows=len(shows), ncols=ncols, pad=pad, **kwargs)
    
    dl.show_batch((xs, ys), ctxs=grid[1::2], show=True)
    show_grid(grid[::2], shows, show=False)
    normalize_axes(grid, xmax=xmax, ymax=ymax)
    
    return fig, grid



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/03_segmentation.core.ipynb.

# %% auto 0
__all__ = ['show_tfms', 'SegmentationExplorer']

# %% ../../nbs/03_segmentation.core.ipynb 3
#TODO: optimize imports
import polvo
from fastai.vision.all import *

# %% ../../nbs/03_segmentation.core.ipynb 11
# TODO: Move DataExplorer base either to common or core
class SegmentationExplorer(polvo.classification.DataExplorer):
    def __init__(self, x_tl, y_tl, id2label):
        self.x_tl = x_tl
        self.y_tl = y_tl
        self.id2label = id2label
        
        self.grouped_y = self._group_y()
        
        
    def _group_y(self):
        label_stats = []
        for item_idx, y in enumerate(progress_bar(self.y_tl)):
            labels, counts = np.unique(y, return_counts=True)
            label_stats.extend([{'item_idx': item_idx,
                                 'label_id': l,
                                 'pct': c / counts.sum(),
                                 'count': c,
                                 'label': self.id2label[l]} 
                                for l, c in zip(labels, counts)])
        
        grouped_y = pd.DataFrame(label_stats)
        grouped_y = grouped_y.groupby("label_id").agg({'label': 'first',
                                                       'item_idx': list,
                                                       'pct': list,
                                                       'count': sum}).reset_index()
        grouped_y["n_samples"] = grouped_y.item_idx.str.len()
        grouped_y = grouped_y.sort_values("count", ascending=False).reset_index(drop=True)
        return grouped_y
    
    def most_common_appearances(self):
        "Return a dafarame ordered by the number of appearances of a label in the dataset."
        return self.grouped_y.sort_values('n_samples', ascending=False).reset_index(drop=True)
    
    def most_common_pixels(self):
        "Return a dataframe ordered by the most common pixels in the dataset."
        return self.most_common()

    @classmethod
    def from_datasets(cls, dss, id2label):
        return cls(dss.tls[0], dss.tls[1], id2label)

# %% ../../nbs/03_segmentation.core.ipynb 20
show_tfms = polvo.show_tfms



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/polvo/',
                'doc_host': 'https://lgvaz.github.io',
                'git_url': 'https://github.com/lgvaz/polvo/',
                'lib_path': 'polvo'},
  'syms': { 'polvo.classification.baseline.baseline': { 'polvo.classification.baseline.baseline.FastaiMetricWrapper': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#fastaimetricwrapper',
                                                                                                                        'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L11'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.__init__': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#__init__',
                                                                                                                                 'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L13'),
                                                        'polvo.classification.baseline.baseline.FastaiMetricWrapper.accumulate': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#accumulate',
                                                                                                                                   'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L16'),
                                                        'polvo.classification.baseline.baseline.calculate_metrics': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#calculate_metrics',
                                                                                                                      'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L21'),
                                                        'polvo.classification.baseline.baseline.fixed_probs': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#fixed_probs',
                                                                                                                'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L37'),
                                                        'polvo.classification.baseline.baseline.random_probs': ( 'https://lgvaz.github.io/polvo/02b_classification.baseline.baseline.html#random_probs',
                                                                                                                 'https://github.com/lgvaz/polvo//blob/master/polvo/classification/baseline/baseline.py#L43')},
            'polvo.classification.explorer': { 'polvo.classification.explorer.DataExplorer': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#dataexplorer',
                                                                                               'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L12'),
                                               'polvo.classification.explorer.DataExplorer.__init__': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#__init__',
                                                                                                        'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L13'),
                                               'polvo.classification.explorer.DataExplorer._group_y': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#_group_y',
                                                                                                        'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L22'),
                                               'polvo.classification.explorer.DataExplorer.from_datasets': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#from_datasets',
                                                                                                             'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L54'),
                                               'polvo.classification.explorer.DataExplorer.most_common': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#most_common',
                                                                                                           'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L32'),
                                               'polvo.classification.explorer.DataExplorer.show_label': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#show_label',
                                                                                                          'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L41'),
                                               'polvo.classification.explorer.DataExplorer.show_label_id': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#show_label_id',
                                                                                                             'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L36'),
                                               'polvo.classification.explorer.DataExplorer.show_most_common': ( 'https://lgvaz.github.io/polvo/02a_classification.explorer.html#show_most_common',
                                                                                                                'https://github.com/lgvaz/polvo//blob/master/polvo/classification/explorer.py#L44')},
            'polvo.common': { 'polvo.common.get_image_grid': ( 'https://lgvaz.github.io/polvo/common.html#get_image_grid',
                                                               'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L13'),
                              'polvo.common.image_grid': ( 'https://lgvaz.github.io/polvo/common.html#image_grid',
                                                           'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L44'),
                              'polvo.common.image_size': ( 'https://lgvaz.github.io/polvo/common.html#image_size',
                                                           'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L53'),
                              'polvo.common.image_sizes_hist': ( 'https://lgvaz.github.io/polvo/common.html#image_sizes_hist',
                                                                 'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L58'),
                              'polvo.common.normalize_axes': ( 'https://lgvaz.github.io/polvo/common.html#normalize_axes',
                                                               'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L37'),
                              'polvo.common.one_batch_with_idxs': ( 'https://lgvaz.github.io/polvo/common.html#one_batch_with_idxs',
                                                                    'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L69'),
                              'polvo.common.show_grid': ( 'https://lgvaz.github.io/polvo/common.html#show_grid',
                                                          'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L32'),
                              'polvo.common.show_tfms': ( 'https://lgvaz.github.io/polvo/common.html#show_tfms',
                                                          'https://github.com/lgvaz/polvo//blob/master/polvo/common.py#L85')},
            'polvo.segmentation.core': { 'polvo.segmentation.core.SegmentationExplorer': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#segmentationexplorer',
                                                                                           'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L13'),
                                         'polvo.segmentation.core.SegmentationExplorer.__init__': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#__init__',
                                                                                                    'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L14'),
                                         'polvo.segmentation.core.SegmentationExplorer._group_y': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#_group_y',
                                                                                                    'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L22'),
                                         'polvo.segmentation.core.SegmentationExplorer.from_datasets': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#from_datasets',
                                                                                                         'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L50'),
                                         'polvo.segmentation.core.SegmentationExplorer.most_common_appearances': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#most_common_appearances',
                                                                                                                   'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L42'),
                                         'polvo.segmentation.core.SegmentationExplorer.most_common_pixels': ( 'https://lgvaz.github.io/polvo/segmentation.core.html#most_common_pixels',
                                                                                                              'https://github.com/lgvaz/polvo//blob/master/polvo/segmentation/core.py#L46')}}}